RicoMiniMap = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
RicoMiniMap:RegisterDB("RicoMiniMapDB")
local dewdrop = AceLibrary("Dewdrop-2.0")

RicoMiniMap_OptionTable = {
	type    = "group",
	handler = RicoMiniMap,
	args = {
		RicoMiniMap_DragLock = {
			type = "toggle",
			order= 1.1,
			name = "Locked",
			desc = "Prevent dragging the map or buttons",
			get  = function() return RicoMiniMap:IsLocked() end,
			set  = function() RicoMiniMap:ToggleLocked() end,
		},
		RicoMiniMap_Coordinates = {
			type = "toggle",
			order= 1.1,
			name = "Show Coordinates",
			desc = "Show your location coordiates",
			get  = function() return RicoMiniMap:IsShowingCoordinates() end,
			set  = function() RicoMiniMap:ToggleShowingCoordinates() end,
		},
		RicoMiniMap_CoordinateTenths = {
			type = "toggle",
			order= 1.2,
			name = "Show Coordinate Tenths",
			desc = "Show your location with tenths",
			get  = function() return RicoMiniMap:IsShowingCoordinateTenths() end,
			set  = function() RicoMiniMap:ToggleShowingCoordinateTenths() end,
		},
		RicoMiniMap_CoordPos={
			type ="group",
			order=1.3, 
			name ="Coordinates Position", 
			desc ="Choose the location of the coordinates",
			args ={
				CoordPos1 = {
					type = "toggle", 
					order= 1, 
					name = "Inside Bottom", 
					desc = "Inside Bottom",
					get  = function() return(RicoMiniMap:GetCoordinatesPosition() == 1) end,
					set  = function() RicoMiniMap:SetCoordinatesPosition(1) RicoMiniMap:UpdateCoordinatesPosition() end
				},
				CoordPos2 = {
					type = "toggle", 
					order= 2, 
					name = "Outside Bottom", 
					desc = "Outside Bottom",
					get  = function() return(RicoMiniMap:GetCoordinatesPosition() == 2) end,
					set  = function() RicoMiniMap:SetCoordinatesPosition(2) RicoMiniMap:UpdateCoordinatesPosition() end
				},
			},
		},
		RicoMiniMap_ZoomButtons = {
			type = "toggle",
			order= 1.4,
			name = "Show Zoom Buttons",
			desc = "Show the zoom in and zoom out buttons",
			get  = function() return RicoMiniMap:IsShowingZoomButtons() end,
			set  = function() RicoMiniMap:ToggleShowingZoomButtons() end,
		},
		RicoMiniMap_MapButton = {
			type = "toggle",
			order= 1.5,
			name = "Show Map Button",
			desc = "Show the world map button",
			get  = function() return RicoMiniMap:IsShowingMapButton() end,
			set  = function() RicoMiniMap:ToggleShowingMapButton() end,
		},
		RicoMiniMap_ZoneLabel = {
			type = "toggle",
			order= 1.6,
			name = "Show Zone Text",
			desc = "Show the default zone text",
			get  = function() return RicoMiniMap:IsShowingZoneText() end,
			set  = function() RicoMiniMap:ToggleShowingZoneText() end,
		},
		RicoMiniMap_Shape={
			type ="group",
			order=1.7, 
			name ="Shape", 
			desc ="Choose the shape of the minimap",
			args ={
				Shape1 = {
					type = "toggle", 
					order= 1, 
					name = "Bottom Left", 
					desc = "Bottom Left corner is rounded",
					get  = function() return(RicoMiniMap:GetShape() == 1) end,
					set  = function() RicoMiniMap:SetShape(1) RicoMiniMap:UpdateShape() end
				},
				Shape2 = {
					type = "toggle", 
					order= 2, 
					name = "Square", 
					desc = "All four corners square",
					get  = function() return(RicoMiniMap:GetShape() == 2) end,
					set  = function() RicoMiniMap:SetShape(2) RicoMiniMap:UpdateShape() end
				},
				Shape3 = {
					type = "toggle", 
					order= 3, 
					name = "Round", 
					desc = "All four corners round",
					get  = function() return(RicoMiniMap:GetShape() == 3) end,
					set  = function() RicoMiniMap:SetShape(3) RicoMiniMap:UpdateShape() end
				},
			},
		},
		RicoMiniMap_Scale = {
			type = "range",
			order = 1.8,
			name = "Scale",
			desc = "Scale the minimap from 50% to 150% in 25% increments",
      max = 1.5, 
      min = 0.5,
      step = 0.25,
      isPercent = true,
      get = "GetMapScale",
      set = "SetMapScale",
		},
		RicoMiniMap_Alpha = {
			type = "range",
			order = 1.9,
			name = "Transparency",
			desc = "Set the transparency of the MiniMap",
			max = 1,
			min = 0.1,
			step = 0.05,
			get = "GetAlpha",
			set = "SetAlpha",
		},
	},
}

RicoMiniMap:RegisterDefaults('profile', {
  	ShowCoordinates = true,
  	ShowTenths = true,
  	ShowZoneText = false,
  	ShowZoomButtons = false,
  	ShowMapButton = false,
  	Scale = 1.25,
  	Shape = 1,
  	CoordinatesPosition = 1,
  	Alpha = 1
	}
)

function RicoMiniMap:OnInitialize()
	self:RegisterEvent("MINIMAP_UPDATE_ZOOM")

	self.slashOptions = {
		name = "RicoMiniMap",
		desc = "A customizable MiniMap",
		type = "group",
		args = {
			config = {
				name = "config",
				type = "execute",
				desc = "Open the config menu.",
				func = function() dewdrop:Open(Minimap) end,
			},
			reset = {
				name = "reset",
				type = "execute",
				desc = "Reset the minimap's position.",
				func = function() MinimapCluster:ClearAllPoints() MinimapCluster:SetPoint("CENTER") end,
			},				
		}
	}

	dewdrop:Register(Minimap,
		'children', function()
			dewdrop:FeedAceOptionsTable(RicoMiniMap_OptionTable)
		end
	)

	if self.db.char.HasMoved == nil then
		MinimapCluster:ClearAllPoints()
		MinimapCluster:SetPoint("TOPRIGHT", UIParent)
		self.db.char.HasMoved = true
	end
	
	MinimapCluster:SetScale(RicoMiniMap:GetMapScale())
	MinimapCluster:SetMovable(true)
	Minimap:RegisterForDrag("LeftButton")
	Minimap:SetScript("OnDragStart", function() self:MapDrag(true) end)
	Minimap:SetScript("OnDragStop", function() self:MapDrag(false) end)
--	MiniMapTrackingFrame:SetScale(0.83)
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetPoint("TOPRIGHT", -20, -25)
	MinimapZoneTextButton:SetFrameStrata("LOW")

	MinimapBorderTop:Hide()
	MinimapToggleButton:Hide()
	GameTimeFrame:Hide()

	self.OnMenuRequest = RicoMiniMap_OptionTable
	self.overrideMenu = true
	self:RegisterChatCommand({"/RicoMiniMap", "/rmm"}, self.slashOptions)

	self.buttons = {
		b1 = MiniMapBattlefieldFrame,
		b2 = MiniMapMeetingStoneFrame,
		b3 = MiniMapWorldMapButton,
		b4 = MiniMapMailFrame,
		b6 = MiniMapTrackingFrame,
		b7 = MinimapZoomIn,
		b8 = MinimapZoomOut
	}
	for _, button in pairs(self.buttons) do
		button:SetMovable(true)
		button:RegisterForDrag("LeftButton")
		button:SetScript("OnDragStart", function() self:ButtonDrag(true) end)
		button:SetScript("OnDragStop", function() self:ButtonDrag(false) end)
	end

	self.mmframes = {
		f1 = QuestWatchFrame,
		f2 = QuestTimerFrame,
		f3 = TicketStatusFrame
	}
	for _, mmframe in pairs(self.mmframes) do
		mmframe:SetMovable(true)
		mmframe:SetClampedToScreen(true)
		mmframe:RegisterForDrag("LeftButton")
		mmframe:SetScript("OnDragStart", function() self:MMFrameDrag(true) end)
		mmframe:SetScript("OnDragStop", function() self:MMFrameDrag(false) end)
	end

	RicoMiniMap:UpdateZoomButtons()
	RicoMiniMap:UpdateShape()
	RicoMiniMap:UpdateCoordinatesPosition()
	RicoMiniMap:UpdateAlpha()
	
	if RicoMiniMap:IsShowingMapButton() then
		MiniMapWorldMapButton:Show()
	else
		MiniMapWorldMapButton:Hide()
	end

	if RicoMiniMap:IsShowingZoneText() then
		MinimapZoneTextButton:Show()
	else
		MinimapZoneTextButton:Hide()
	end
end

function RicoMiniMap_OnLoad()
end

function RicoMiniMap:MapDrag(moving)
	if moving and RicoMiniMap:IsLocked() == false then
		MinimapCluster.isMoving = true
		MinimapCluster:StartMoving()
	elseif MinimapCluster.isMoving then
		MinimapCluster.isMoving = false
		MinimapCluster:StopMovingOrSizing()
	end
end

function RicoMiniMap:ButtonDrag(moving)
	local function getPos()
		local mapShape = RicoMiniMap:GetShape()
		local cx, cy = GetCursorPosition(UIParent)
		local mx, my = Minimap:GetLeft(), Minimap:GetBottom()
		local z = Minimap:GetEffectiveScale()
		local vector = (math.deg(math.atan2(cy / z - my - 70, mx - cx / z + 70)))
		if mapShape == 3 then
			x = 52 - 81 * cos(vector)
			y = 81 * sin(vector) - 54
		else
			--DEFAULT_CHAT_FRAME:AddMessage(mapShape.."/"..vector)
			x = 52 - math.max(-81, math.min(110 * cos(vector), 81))
			y = math.max(-81, math.min(110 * sin(vector), 81)) - 54
			if mapShape == 1 and vector > -90 and vector < 0 then
				x = 52 - 81 * cos(vector)
				y = 81 * sin(vector) - 54
			end
		end
		return x, y
	end
	if moving and RicoMiniMap:IsLocked() == false then
		this.isMoving = true
		this:SetScript("OnUpdate", function()
			local x, y = getPos()
      		this:ClearAllPoints()
			this:SetPoint("TOPLEFT", Minimap, "TOPLEFT", x, y)
		end)
		this:StartMoving()
	elseif this.isMoving then
		this.isMoving = false
		this:StopMovingOrSizing()
		this:SetScript("OnUpdate", nil)
	end
end

function RicoMiniMap:MMFrameDrag(moving)
	if moving and RicoMiniMap:IsLocked() == false then
		this.isMoving = true
		this:StartMoving()
	elseif this.isMoving then
		this.isMoving = false
		this:StopMovingOrSizing()
	end
end

function RicoMiniMap:IsLocked()
	if self.db.profile.Locked == nil then
		self.db.profile.Locked = false
	end
	return self.db.profile.Locked
end

function RicoMiniMap:ToggleLocked()
	self.db.profile.Locked = not self.db.profile.Locked
	return self.db.profile.Locked
end

function RicoMiniMap:IsShowingCoordinates()
	if self.db.profile.ShowCoordinates == nil then
		self.db.profile.ShowCoordinates = true
	end
	return self.db.profile.ShowCoordinates
end

function RicoMiniMap:ToggleShowingCoordinates()
	self.db.profile.ShowCoordinates = not self.db.profile.ShowCoordinates
	return self.db.profile.ShowCoordinates
end

function RicoMiniMap:IsShowingCoordinateTenths()
	if self.db.profile.ShowTenths == nil then
		self.db.profile.ShowTenths = true
	end
	return self.db.profile.ShowTenths
end

function RicoMiniMap:ToggleShowingCoordinateTenths()
	self.db.profile.ShowTenths = not self.db.profile.ShowTenths
	return self.db.profile.ShowTenths
end

function RicoMiniMap:IsShowingZoomButtons()
	if self.db.profile.ShowZoomButtons == nil then
		self.db.profile.ShowZoomButtons = false
	end
	return self.db.profile.ShowZoomButtons
end

function RicoMiniMap:ToggleShowingZoomButtons()
	self.db.profile.ShowZoomButtons = not self.db.profile.ShowZoomButtons
	RicoMiniMap:UpdateZoomButtons()
	return self.db.profile.ShowZoomButtons
end

function RicoMiniMap:UpdateZoomButtons()
	if RicoMiniMap:IsShowingZoomButtons() then
		MinimapZoomIn:Show()
  	MinimapZoomOut:Show()	
	else
		MinimapZoomIn:Hide()
  	MinimapZoomOut:Hide()	
	end
end

function RicoMiniMap:IsShowingMapButton()
	if self.db.profile.ShowMapButton == nil then
		self.db.profile.ShowMapButton = false
	end
	return self.db.profile.ShowMapButton
end

function RicoMiniMap:ToggleShowingMapButton()
	self.db.profile.ShowMapButton = not self.db.profile.ShowMapButton
	if RicoMiniMap:IsShowingMapButton() then
		MiniMapWorldMapButton:Show()
	else
		MiniMapWorldMapButton:Hide()
	end
	return self.db.profile.ShowMapButton
end

function RicoMiniMap:IsShowingZoneText()
	if self.db.profile.ShowZoneText == nil then
		self.db.profile.ShowZoneText = false
	end
	return self.db.profile.ShowZoneText
end

function RicoMiniMap:ToggleShowingZoneText()
	self.db.profile.ShowZoneText = not self.db.profile.ShowZoneText
	if RicoMiniMap:IsShowingZoneText() then
		MinimapZoneTextButton:Show()
	else
		MinimapZoneTextButton:Hide()
	end
	return self.db.profile.ShowZoneText
end

function RicoMiniMap:GetMapScale()
	if self.db.profile.Scale == nil then
		self.db.profile.Scale = 1.25
	end
	return self.db.profile.Scale
end

function RicoMiniMap:SetMapScale(value)
	MinimapCluster:SetScale(value)
	self.db.profile.Scale = value
end

function RicoMiniMap:GetAlpha()
	if self.db.profile.Alpha == nil then
		self.db.profile.Alpha = 1
	end
	return self.db.profile.Alpha
end

function RicoMiniMap:SetAlpha(value)
	self.db.profile.Alpha = value
	RicoMiniMap:UpdateAlpha()
end

function RicoMiniMap:UpdateAlpha()
	local value = self.db.profile.Alpha
	if value ~= nil then
		Minimap:SetAlpha(value)
	end
end

function RicoMiniMap:GetShape()
	if self.db.profile.Shape == nil then
		self.db.profile.Shape = 1
	end
	return self.db.profile.Shape
end

function RicoMiniMap:SetShape(value)
	self.db.profile.Shape = value
end

function RicoMiniMap:UpdateShape()
	local value = self.db.profile.Shape
	local texture = "Interface\\Minimap\\UI-Minimap-Border"
	local mask = "textures\\MinimapMask"
	
	if value == 1 then
		texture = "Interface\\AddOns\\RicoMiniMap\\BLRoundMinimap"
		mask = "Interface\\AddOns\\RicoMiniMap\\BLRoundMinimapMask"
	end
	
	if value == 2 then
		texture = "Interface\\AddOns\\RicoMiniMap\\SquareMinimap"
		mask = "Interface\\AddOns\\RicoMiniMap\\SquareMinimapMask"
	end
	
	MinimapBorder:SetTexture(texture)
	MinimapBorder:SetDrawLayer("ARTWORK")
	Minimap:SetMaskTexture(mask)
	RicoMiniMap:UpdateCoordinatesPosition()
end

function RicoMiniMap:GetCoordinatesPosition()
	if self.db.profile.CoordinatesPosition == nil then
		self.db.profile.CoordinatesPosition = 1
	end
	return self.db.profile.CoordinatesPosition
end

function RicoMiniMap:SetCoordinatesPosition(value)
	self.db.profile.CoordinatesPosition = value
end

function RicoMiniMap:UpdateCoordinatesPosition()
	local value = self.db.profile.CoordinatesPosition
	RicoMinimap_CoordinatesFrame:ClearAllPoints()
	if value == 1 then
		if self.db.profile.Shape == 1 then
			RicoMinimap_CoordinatesFrame:SetPoint("BOTTOM", "Minimap", "BOTTOM", 30, 0)
		else
			RicoMinimap_CoordinatesFrame:SetPoint("BOTTOM", "Minimap", "BOTTOM", 0, 0)
		end
		RicoMinimap_CoordinatesFrame:SetBackdropColor(0, 0, 0, 0)
		RicoMinimap_CoordinatesFrame:SetBackdropBorderColor(0, 0, 0, 0)
	else
		if self.db.profile.Shape == 1 then
			RicoMinimap_CoordinatesFrame:SetPoint("TOP", "Minimap", "BOTTOM", 30, 0)
		else
			RicoMinimap_CoordinatesFrame:SetPoint("TOP", "Minimap", "BOTTOM", 0, 0)
		end
		RicoMinimap_CoordinatesFrame:SetBackdropColor(0, 0, 0, 1)
		RicoMinimap_CoordinatesFrame:SetBackdropBorderColor(0, 0, 0, 1)
	end
end


------------------
-- Coordinates text
function RicoMiniMap_updateLocText()
	if RicoMiniMap:IsShowingCoordinates() then
		RicoMinimap_CoordinatesFrame:Show()
		local x, y = GetPlayerMapPosition('player')
		if x == 0 and y == 0 then 
			RicoMinimap_CoordinatesText:SetText('')
		else
			if RicoMiniMap:IsShowingCoordinateTenths() then
				RicoMinimap_CoordinatesText:SetText(string.format('%.1f, %.1f', x*100, y*100))
			else
				RicoMinimap_CoordinatesText:SetText(string.format('%.0f, %.0f', x*100, y*100))
			end
		end
		RicoMinimap_CoordinatesFrame:SetWidth(RicoMinimap_CoordinatesText:GetWidth() + 20)
		RicoMinimap_CoordinatesFrame:SetHeight(RicoMinimap_CoordinatesText:GetHeight() + 12)
	else
		RicoMinimap_CoordinatesFrame:Hide()
	end
end

-------------------------------
-- Zooming with the mouse wheel
function MiniMap_OnMouseWheel(value)
		if value > 0 then
			RicoMinimap_ZoomIn()
		elseif value < 0 then
			RicoMinimap_ZoomOut()
		end
end

function RicoMinimap_ZoomIn()
	if Minimap:GetZoom() == 5 then
		Minimap:SetZoom(5)
	else
		Minimap:SetZoom(Minimap:GetZoom() + 1)
	end
end

function RicoMinimap_ZoomOut()
	if Minimap:GetZoom() == 0 then
		Minimap:SetZoom(0)
	else
		Minimap:SetZoom(Minimap:GetZoom() - 1)
	end
end

-----------------
-- Event handlers
function RicoMiniMap:MINIMAP_UPDATE_ZOOM()
	if (tonumber(GetCVar("minimapInsideZoom")) == Minimap:GetZoom()) then
		self.inside = true
	else
		self.inside = false
	end
	if(self.inside) then
		Minimap:SetAlpha(1)
	else
		Minimap:SetAlpha(self.db.profile.Alpha)
	end
end
